
package org.gjt.bugrat.dbi;

import java.sql.*;
import java.util.*;
import java.net.URL.*;

import org.gjt.bugrat.util.BugRatLogger;


public class
JDBCHandler
	{
	private boolean			isOpen = false;
	private Connection		connection = null;
	private Class			driverClass = null;
	private BugRatLogger	logger = null;
	private boolean			performEscapeProcessing = false;
	private String			driverPrefix = null;
	private String			driverClassName = null;
	private boolean			usingOracle = false;


	/**
	 * We save the URL, userName, and password used to open the
	 * connection so we can reopen when the connection times out.
	 */
	private String			jdbcUrl = null;
	private Properties		jdbcProps = null;
	
	private JDBCHandlerThread	jcvsThread = null;


	private
	JDBCHandler()
		throws ClassNotFoundException
		{
		throw new RuntimeException
			( "constructor JDBCHandler() is not public" );
		}

	public
	JDBCHandler(
			String drvrClsName, String jdbcUrl,
			Properties jdbcProps, BugRatLogger logger )
		throws ClassNotFoundException
		{
		this.jdbcUrl = jdbcUrl;
		this.jdbcProps = jdbcProps;

		this.driverClassName = drvrClsName;
		this.driverClass = Class.forName( this.driverClassName );

		//
		// NOTE We never use escape's, but '{' often turns up in
		//      comments, causing the SQL to fail. Thus, we turn
		//      off escape processing.
		//
		this.performEscapeProcessing = false;

		String oracleStr = (String)
			this.jdbcProps.get( "jdbcUsingOracle" );

		if ( oracleStr != null
			&&  (  oracleStr.equalsIgnoreCase( "on" )
				|| oracleStr.equalsIgnoreCase( "yes" )
				|| oracleStr.equalsIgnoreCase( "true" ) ) )
			{
			this.usingOracle = true;
			}

		String escStr = (String)
			this.jdbcProps.get( "jdbcEscapeProcessing" );

		if ( escStr != null
			&&  (  escStr.equalsIgnoreCase( "on" )
				|| escStr.equalsIgnoreCase( "yes" )
				|| escStr.equalsIgnoreCase( "true" ) ) )
			{
			this.performEscapeProcessing = true;
			}
		}

	public void
	openConnection()
		throws SQLException
		{
		this.performOpen( jdbcUrl, jdbcProps );

		String kaStr = (String)
			this.jdbcProps.get( "jdbcKeepAliveThread" );

		if ( kaStr != null &&
				( kaStr.equalsIgnoreCase( "true" )
					|| kaStr.equalsIgnoreCase( "on" )
					|| kaStr.equalsIgnoreCase( "yes" ) ) )
			{
			this.jcvsThread =
				new JDBCHandlerThread( this, this.logger );
			this.jcvsThread.start();
			}
		}

	public void
	reopenConnection()
		throws SQLException
		{
		if ( this.isOpen() )
			{
			this.closeConnection();
			}

		this.performOpen( jdbcUrl, jdbcProps );
		}

	protected void
	performOpen( String jdbcUrl, Properties jdbcProps )
		throws SQLException
		{
		if ( this.isOpen() )
			throw new SQLException
				( "Connection is already open" );

		try {
			Driver driver = null;

			// NOTE
			// We have to enumerate the drivers here, since if we
			// were to call DriverManager.getConnection() with a
			// standard prefix (say 'mysql'), any number of drivers
			// could answer for it. Since the manager will simply
			// accept the first one that responds, we may get the
			// wrong driver and not be able to get the one we want.
			// Thus, we force the match on the driver classname.
			//
			Enumeration drivers = DriverManager.getDrivers();
			for ( ; drivers.hasMoreElements() ; )
				{
				Driver d = (Driver) drivers.nextElement();
				if ( d.getClass() == this.driverClass )
					{
					driver = d;
					break;
					}
				}

			//
			// First, if the driver was found by its class name, try to
			// open the connection using the instance we found...
			//
			if ( driver != null )
				{
				this.connection = driver.connect( jdbcUrl, jdbcProps );
				}

			//
			// Then, if the connection was not established by calling
			// the driver (or not if it was not found), we will use
			// the more general DriverManager connection method.
			//
			if ( this.connection == null )
				{
				this.connection =
					DriverManager.getConnection( jdbcUrl, jdbcProps );
				}

			if ( this.connection == null )
				throw new SQLException
					( "could not establish the JDBC connection" );

			this.isOpen = true;
			}
		catch ( SQLException ex )
			{
			this.isOpen = false;
			this.connection = null;
			throw ex;
			}
		}

	public void
	closeConnection()
		{
		if ( this.isOpen() && this.connection != null )
			{
			try { this.connection.close(); }
			catch ( SQLException ex )
				{
				this.logger.log( "closing sql connection", ex );
				}
			}

		this.isOpen = false;
		}		

	public Statement
	createStatement()
		throws SQLException
		{
		if ( ! this.isOpen() )
			throw new SQLException
				( "SQLHandler connection is not open" );

		Statement stmnt = this.connection.createStatement();

		stmnt.setEscapeProcessing( this.performEscapeProcessing );

		return stmnt;
		}
    
	public PreparedStatement
	createPreparedStatement( String statement )
		throws SQLException
		{
		if ( ! this.isOpen() )
			throw new SQLException
				( "SQLHandler connection is not open" );

		PreparedStatement ps =
			this.connection.prepareStatement( statement );

		ps.setEscapeProcessing( this.performEscapeProcessing );

		return ps;
		}

	public boolean
	isOpen()
		{
		return this.isOpen;
		}

	public boolean
	isUsingOracle()
		{
		return this.usingOracle;
		}

	}


